#!/bin/bash

#
# Copyright (C) mailscanner.sh  Markus Madlener
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * #
# *                                                                 * #
# *                   -->  W A R N I N G  <--                       * #
# *                                                                 * #
# *  This file may only be edited on a UNIX/LINUX box using an      * #
# *  adequate text editor like sed, vi or the like.                 * #
# *                                                                 * #
# *  It contains special control characters that are invisible in   * #
# *  most text editors but absolutly essential for this files       * #
# *  correct functioning.                                           * #
# *  Especially when edited on a different platform (ex. Windows)   * #
# *  it is evident that these characters are stripped from the      * #
# *  file without notice even if the editor knows about the         * #
# *  different conventions of the involved platforms.               * #
# *  This is not a cosmetic problem!                                * #
# *                                                                 * #
# *  Violation of this "law of editing" might avenge misbehaviour   * #
# *  of all kind, including (but not limited to) any kind of        * #
# *  slight disfunction in rare cases, data loss, infinite loops,   * #
# *  not running at all and so on.                                  * #
# *  The resulting errors might be extremely difficult to track     * #
# *  down to their source.                                          * #
# *                                                                 * #
# *  Think twice before you proceed and always have a backup handy! * #
# *                                                                 * #
# *  If this warning box doesn't appear as a well formed box and    * #
# *  the lines seem to be wrapped accidently, then something went   * #
# *  seriously wrong!                                               * #
# *  Then this file isn't usable any more and should be replaced    * #
# *  by a working backup copy immediately!                          * #
# *                                                                 * #
# * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # 
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # Version 2013-07-13_v2_beta4_01                              # # #
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #

. /var/log/copfilter/default/etc/global_settings
. $BASEDIR/etc2/copfilter_functions
. /var/ipcop/main/settings
. $BASEDIR/langs/en.sh
. $BASEDIR/langs/${LANGUAGE}.sh

# DATE=`date +%Y%m%d%H%M%S`
LOGFILE=`mktemp $TMPDIR/mailscanner_sh.log_XXXXXX`

exec 7>&1
exec > $LOGFILE




returncode_scanner=0

# UNIQ_FILE=`date +%Y%m%d%H%M%S%N`
TIMEBIN=${BASEDIR}/opt/tools/bin/time
CLAMD_SCANNED=no
AVG_SCANNED=no
FPROT_SCANNED=no
AVAST_SCANNED=no
RENATTACH_SCANNED=no
SPAMD_SCANNED=no
WHITELIST_MODE=no
# WH_MAIL_RESULT=`mktemp $TMPDIR/copfilter_whitelist_result.XXXXXX`
LOCALNET=no
LOCALNET2=no
LOCALCHECK=yes
# TEMPLIST=`mktemp $TMPDIR/local_webgui.cf.XXXXXX`

echo $EMAIL

if [ "x${EMAIL}" = "x" ]; then 
    CURRENT_SCAN=POP3
    EMAIL="${1}"
    MAILFROM="${2}"
    MAILTO="${3}"
    USERNAME="${4}"
    SUBJECT="${5}"
    MAILDATE="${6}"
    SERVERIP="${7}"
    SERVERPORT="${8}"
    CLIENTIP="${9}"
    CLIENTPORT="${10}"
    PROTOCOL="${11}"
    PROGNAME="${12}"
    VERSIONPROG="${13}"
    VDINFO="${14}"
else
   CURRENT_SCAN=SMTP
fi

# Added JM because of issues when /dev/root is full (emails get lost)
FREESPACE=`df | grep da1 | awk '{print $4}'`
if [ "$FREESPACE" -lt "40960" ]; then
        if [ "$CURRENT_SCAN" = "SMTP" ]; then
                echo "452 Not enough space available for processing the request"
                echo "452 Not enough space available for processing the request" >&2
        fi
        exec 1>&7 7>&-
        cat $LOGFILE
        rm -f $LOGFILE
        cleanup
        cleanup_tmp_files
	$SENDEMAIL -xu $SMTP_AUTH_USER -xp $SMTP_AUTH_PASS \
                 -f $SENDER_ADR -t $SENDER -s "${SMTP_HOST}"  \
                 -u "[Copfilter] root filesystem is running out of space, please check" -m "Performing automated fixup..."
        $BASEDIR/opt/tools/bin/check_status.sh &
        exit 1
fi

EMAILID=`echo $EMAIL | sed -e "s,.*/\(.*\)$,\1,"`
echo EMAILID $EMAILID

echo "Original SENDER address is $SENDER"

# Clean-up Sender-Address if required
# Check if Email-Address is only a SMTP-Address
if [ $CURRENT_SCAN = "POP3" ];then
	SENDER=$MAILFROM
fi
RESULT=`echo $SENDER | grep -v -i -e '^[A-Z0-9._%+-]\+@[A-Z0-9._%+-]\+.[A-Z]$'`
echo $RESULT &>/dev/null
SENDER_COMPL=$?
if [ "x${SENDER_COMPL}" = "x0" ]; then
    # Remove quotes and double-quotes from sender-string
    SENDER=`echo $SENDER | sed -e "s/'//g" | sed -e 's/"//g'`
    # Trim Sender String so that only Email-Address is stored
    # Check type of brackets used
    echo $SENDER | grep "<" &>/dev/null
    BR_TYPE=$?
    if [ "x${BR_TYPE}" = "x0" ]; then
	SENDER=`echo $SENDER | cut -d "<" -f 2 | cut -d ">" -f 1`
    else
	echo $SENDER | grep "\[" &>/dev/null
	BR_TYPE=$?
    	if [ "x${BR_TYPE}" = "x0" ]; then
    		SENDER=`echo $SENDER | cut -d "[" -f 2 | cut -d "]" -f 1`
    	fi
    fi
    # Check if more than one sender address is specified, use only the first one. Issue rasied here: http://copfilter.endlich-mail.de/viewtopic.php?p=9169&highlight=#9169
    ADDRCOUNT=`echo $SENDER | grep -i "@" | wc -w`
    if [ "$ADDRCOUNT" -gt "1" ]; then
	SENDER=`echo $SENDER | cut -d " " -f 1`
    fi
fi

echo "Clean SENDER address is $SENDER"
#testing
#CURRENT_SCAN=SMTP
#cp $EMAIL $TMPDIR/test.$$
#echo "backuped email for later debugging:  $TMPDIR/test.$$"
#
#echo CURRENT_SCAN   $CURRENT_SCAN >$TMPDIR/env.p3scan
#set >>$TMPDIR/set.dd
#echo EMAIL          $EMAIL >>$TMPDIR/env.p3scan
#echo EMAIL          $EMAIL 
#FF=`echo $EMAIL |sed -e "s,\(.*[0-9][0-9]\+\)/.*,\1,"`
#echo XXXX >${FF}/vnmsg
#ls -la ${FF}/vnmsg
#cat ${FF}/vnmsg
#echo MAILFROM       $MAILFROM >>$TMPDIR/env.p3scan
#echo MAILTO         $MAILTO >>$TMPDIR/env.p3scan
#echo USERNAME       $USERNAME >>$TMPDIR/env.p3scan
#echo SUBJECT        $SUBJECT >>$TMPDIR/env.p3scan
#echo MAILDATE       $MAILDATE >>$TMPDIR/env.p3scan
#echo SERVERIP       $SERVERIP >>$TMPDIR/env.p3scan
#echo SERVERPORT     $SERVERPORT >>$TMPDIR/env.p3scan
#echo CLIENTIP       $CLIENTIP >>$TMPDIR/env.p3scan
#echo CLIENTPORT     $CLIENTPORT >>$TMPDIR/env.p3scan
#echo PROTOCOL       $PROTOCOL >>$TMPDIR/env.p3scan
#echo PROGNAME       $PROGNAME >>$TMPDIR/env.p3scan
#echo VERSION        $VERSION >>$TMPDIR/env.p3scan
#echo VDINFO         $VDINFO >>$TMPDIR/env.p3scan

cleanup () {

	# deleting temporary files and dirs
	rm -rf $EMAIL.dir
#    rm -f $TEMPLIST
}


if [ "x${CP_WH_CMD_MODE}" = "xon" ];then

echo "------------------------------------------------------------------"
echo "Copfilter whitelist command mail mode "
echo

if [ $CURRENT_SCAN = "SMTP" ]; then
 # allow mailcommand mode only on local network
 if [ "x${SERVERIP}" != "x" ]; then
      LOCALCHECK=`$BASEDIR/opt/tools/bin/IpInSubnet.sh $SERVERIP`
 fi
 
 if [ "$LOCALCHECK" = "yes" ]; then

 ADD_TO_WHITELIST=`mktemp $TMPDIR/copfilter_add_to_whitelist.XXXXXX`
 REMOVE_FROM_WHITELIST=`mktemp $TMPDIR/copfilter_remove_from_whitelist.XXXXXX`

 grep -i "copfilter_add_to_whitelist" $EMAIL > $ADD_TO_WHITELIST
 ADD_TO_WHITELIST_MODE=$?

 grep -i "copfilter_remove_from_whitelist" $EMAIL > $REMOVE_FROM_WHITELIST
 REMOVE_FROM_WHITELIST_MODE=$?


    WH_MAIL_RESULT=`mktemp $TMPDIR/copfilter_whitelist_result.XXXXXX`

    echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-" > $WH_MAIL_RESULT
    echo >> $WH_MAIL_RESULT
    echo "Copfilter whitelist management" >> $WH_MAIL_RESULT
    echo "Command results" >> $WH_MAIL_RESULT
    echo >> $WH_MAIL_RESULT
    echo >> $WH_MAIL_RESULT



    if [ "x${ADD_TO_WHITELIST_MODE}" = "x0" ];then
        WHITELIST_MODE=yes
        RECIPIENTS=`cat $ADD_TO_WHITELIST | sed -e :a -e '/./N;  s/\n/ /;ta' -e 's/[~!#%&^*?;()<>{}"\/;]//g' \
           -e "s/'//g" -e 's/\[//g' -e 's/\]//g' | /usr/bin/awk -f $BASEDIR/opt/tools/bin/extract_email_adresses.awk`
        RECIPIENTS2=`echo $RECIPIENTS | sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`
        echo "Email addresses to add: $RECIPIENTS2"
        for i in $RECIPIENTS2
        do
             ii=`echo $i |sed -n 's/\([a-z0-9][a-z0-9._+-]*@[.a-z0-9_+-]\+[.]\+[a-z][a-z]\+\).*/\1/p'`
             grep -i "whitelist_from $ii" $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf &>/dev/null
             RESULT1=$?
             if [   "x${RESULT1}" != "x0" -a "x${ii}" != "x" ];then
                  echo "whitelist_from $ii" >> $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf
                  echo "added $ii to whitelist" |tee -a $WH_MAIL_RESULT
             else
                  echo "$i is already in whitelist, or is a wrong email address"  |tee -a $WH_MAIL_RESULT
             fi
        done
    fi



    if [ "x${REMOVE_FROM_WHITELIST_MODE}" = "x0" ];then
        WHITELIST_MODE=yes
        RECIPIENTS=`cat $REMOVE_FROM_WHITELIST | sed -e :a -e '/./N;  s/\n/ /;ta' -e 's/[~!#%&^*?;()<>{}"\/;]//g' \
           -e "s/'//g" -e 's/\[//g' -e 's/\]//g'| /usr/bin/awk -f $BASEDIR/opt/tools/bin/extract_email_adresses.awk`
        RECIPIENTS2=`echo $RECIPIENTS | sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`
        echo "Email addresses to remove: $RECIPIENTS2"
        for i in $RECIPIENTS2
          do
             ii=`echo $i |sed -n 's/\([a-z0-9][a-z0-9._+-]*@[.a-z0-9_+-]\+[.]\+[a-z][a-z]\+\).*/\1/p'`
             grep -i "whitelist_from $ii" $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf &>/dev/null
             RESULT1=$?
             if [   "x${RESULT1}" != "x0" -o "x${ii}" = "x" ];then
                  echo "$i is not in whitelist, or is a wrong email address" |tee -a $WH_MAIL_RESULT
             else
                  TMPFILE=`mktemp $TMPDIR/local_webgui.cf.XXXXXX`
                  cat $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf |sed -e "/$ii/d" > $TMPFILE
                  mv $TMPFILE $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf
                  echo "removed $ii from whitelist" |tee -a $WH_MAIL_RESULT
             fi
        done

    fi



    if [ "x${WHITELIST_MODE}" = "xyes" ];then
        if [ -f $BASEDIR/etc/smtp_auth_enable ]; then
             echo "smtp authentication enabled, using user: $SMTP_AUTH_USER"
             cat $WH_MAIL_RESULT | $SENDEMAIL -xu $SMTP_AUTH_USER -xp $SMTP_AUTH_PASS \
                 -f $SENDER_ADR -t $SENDER -s "${SMTP_HOST}"  \
                 -u "[Copfilter] Whitelist Management" |logger -t "sendemail cp whitelist mgmt"
        else
             cat $WH_MAIL_RESULT | $SENDEMAIL -f $SENDER_ADR -t $SENDER -s "${SMTP_HOST}"  \
                 -u "[Copfilter] Whitelist Management" |logger -t "sendemail cp whitelist mgmt"

        fi

        rm -f $WH_MAIL_RESULT
        rm -f $ADD_TO_WHITELIST $REMOVE_FROM_WHITELIST
        echo "250 Content Discarded: Copfilter whitelist command"
        echo "250 Content Discarded: Copfilter whitelist command" >&2

        exec 1>&7 7>&-
        cat $LOGFILE

        rm -f $LOGFILE
        cleanup
        cleanup_tmp_files
        exit 1
    fi
    echo "is local $SERVERIP $MERK $LOCALCHECK" >> /tmp/mailscanner_log
 else
     echo "not local ServerIP $SERVERIP ClientIP $CLIENTIP" >> /tmp/mailscanner_log
 fi

  fi
  rm -f $ADD_TO_WHITELIST $REMOVE_FROM_WHITELIST
  rm -f $WH_MAIL_RESULT
else
    echo "command mail mode is off"
fi

############################ By pass if Mail is larger than 2M
SIZE_MAIL=0
stat $EMAIL.dir -c%s > /tmp/SIZE_MAIL 2>&1
SIZE_MAIL1=`cat /tmp/SIZE_MAIL`
echo "checking size"
SCANMAIL=0
SMTP_SIZE_MAIL=2000000
if [ $CURRENT_SCAN = "SMTP" ] && [ $SMTP_VIRUS_SCAN_ATTACHMENT = "off" ] && [ "$SIZE_MAIL1" -ge "$SMTP_SIZE_MAIL" ]; then
   echo " scanning outgoing mail is disable because the size of the mail"
else
#############################



echo ------------------------------------------------------------------
echo "decoding email, starting ripmime... "
echo $EMAIL
echo into 
echo $EMAIL.dir 
echo 

#TIME="\nProcess duration: %e seconds" $TIMEBIN $RIPMIME -i $EMAIL -d $EMAIL.dir -v
$RIPMIME -i $EMAIL -d $EMAIL.dir -v

echo 
echo ripmime done !



if [ "$CURRENT_SCAN" = "SMTP" ]; then
     #checks if CLIENT is part of our network
     echo
     echo
	 echo ------------------------------------------------------------------
	 echo "Network check"
     echo
     echo CLIENT is $CLIENT
     echo SERVER is $SERVER
     if [ "x${CLIENT}" != "x" ]; then
          LOCALNET=`$BASEDIR/opt/tools/bin/IpInSubnet.sh $CLIENT`
     fi
     if [ "x${LOCALNET}" = "xyes" ]; then
         LOCALNET2=yes
     fi
fi



# add comment to email header 
if [ "x$POP3_MAIL_ADD_HEADER" = "xon" -a "x$CURRENT_SCAN" = "xPOP3" -o  "x$SMTP_MAIL_ADD_HEADER" = "xon" -a "x$CURRENT_SCAN" = "xSMTP" ]; then
     echo
     echo
     echo ------------------------------------------------------------------
     echo "adding copfilter header toemail ..."
     echo
	
     mail_add_header  $EMAIL 0
	
     echo "adding copfilter header toemail done"
     echo
     echo
else
     echo adding header to email disabled
fi


# Removed to extract the email address from the email rather than relying on Proxsmtpd to deliver the Sender
# if [ "x${CURRENT_SCAN}" = "xPOP3" ];then
     SENDER2=`cat $EMAIL |sed -n '1,/^$/p' | $BASEDIR/opt/tools/bin/formail -b -f -c | /usr/bin/awk -f $BASEDIR/opt/tools/bin/extract_email_adresses_from.awk | /usr/bin/awk '{print $1}'`
     SENDER=`echo $SENDER2| sed -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/'`
# fi

IS_SPAM=0


if [ $CURRENT_SCAN = "POP3" -a $POP3_VIRUS_SCAN = "on" -o $CURRENT_SCAN = "SMTP" -a $SMTP_VIRUS_SCAN = "on" ]; then

 ################ check size outgoing mail to by pass filtering if larger than 2M ################
 SIZE_MAIL1=0
 stat $EMAIL.dir -c%s > /tmp/SIZE_MAIL 2>&1
 SIZE_MAIL1=`cat /tmp/SIZE_MAIL`
 echo "check size"
 SCANMAIL=0
 SMTP_SIZE_MAIL=2000000
 if [ $CURRENT_SCAN = "SMTP" ]; then
    SCANMAIL=`expr $SCANMAIL + 1`  
 fi
 if [ $SMTP_VIRUS_SCAN_ATTACHMENT = "off" ]; then
    SCANMAIL=`expr $SCANMAIL + 1`
 fi
 if [ "$SIZE_MAIL1" -ge "$SMTP_SIZE_MAIL" ]; then
   SCANMAIL=`expr $SCANMAIL + 1`
 fi

 if [ "$SCANMAIL" -eq "3" ]; then
        echo "scanning outgoing mail disable because size larger than 2M"
 else
 #########################

	
        if [ "x${CLAMD_ENABLE}" = "xon"  ]; then


    echo
    echo
	echo ------------------------------------------------------------------
	echo "scanning email for viruses, starting clamscan..."
	echo
	
	# return code 1 is a virus
	#$CLAMSCAN --stdout --tempdir=$TMPDIR --recursive --max-recursion=10 $EMAIL.dir >$TMPDIR/clamscan_output_$UNIQ_FILE.tmp
        #TIME="\nProcess duration: %e seconds" $TIMEBIN $CLAMSCAN --stdout $EMAIL.dir >$TMPDIR/clamscan_output_$UNIQ_FILE.tmp 2>&1 || IS_VIRUS=$?
	returncode_scanner_now=$returncode_scanner
	TMPFILE=`mktemp $TMPDIR/clamscan_output_XXXXXX`
        $CLAMSCAN --fdpass --stdout $EMAIL.dir > $TMPFILE || IS_VIRUS=$?
        echo "return code clamav is $IS_VIRUS"
	if [ "$IS_VIRUS" = 1 ]; then
        	returncode_scanner=3
		virusfile_clam=`cat $TMPFILE|grep FOUND|sed -e 's/.*\/\(.*\): \(.*\) FOUND$/\1/'`
		virusname_clam=`cat $TMPFILE|grep FOUND|sed -e 's/.*\/\(.*\): \(.*\) FOUND$/\2/'`
		if [ "x$virusname_clam" != "x" ]; then
			echo Infection: ClamAV found \*$virusname_clam\* in \*$virusfile_clam\*
		else
			IS_VIRUS=0
			returncode_scanner=$returncode_scanner_now
		fi
	else
		cat $TMPFILE
	fi
	
        CLAMD_SCANNED=yes	

	/bin/rm -f $TMPFILE
	
	echo 
	echo clamscan done

   else
	echo clamd disabled
   fi

   if [ "x${AVG_ENABLE}" = "xon"  ]; then

        if [ "x$AVG_INSTALLED" = 'xyes' ]; then


            echo
            echo
                echo ------------------------------------------------------------------
                echo "scanning email for viruses, starting avg..."
                echo

        # return code 5 is a virus,
        # return code 5 is virus found by heuristic
                # return code 4 is a suspicous file found by heuristic

                #TIME="\nProcess duration: %e seconds" $TIMEBIN $AVG $EMAIL.dir > $TMPDIR/avg_output_$UNIQ_FILE.tmp 2>&1 || IS_VIRUS=$?
		returncode_scanner_now=$returncode_scanner
		TMPFILE=`mktemp $TMPDIR/avg_output_XXXXXX`
#               LD_LIBRARY_PATH=$BASEDIR/opt/avg/lib $BASEDIR/$AVG $EMAIL.dir > $TMPFILE || IS_VIRUS=$?
		avgscan -a $EMAIL.dir > $TMPFILE || IS_VIRUS=$?
                echo "return code avg is $IS_VIRUS"
                if [ "$IS_VIRUS" = 5 ]; then
                        cat $TMPFILE
                        returncode_scanner=3
                        virusfile_avg=`cat $TMPFILE|grep "Virus iden" | sed -e 's/.*\/\(.*\) Virus identified.*/\1/'`
                        virusname_avg=`cat $TMPFILE|grep "Virus iden" | sed -e 's/.*Virus identified \(.*\)$/\1/'`
			if [ "x$virusname_avg" != "x" ]; then
                       		echo Infection: avg found \*$virusname_avg\* in \*$virusfile_avg\*
			else
				IS_VIRUS=0
				returncode_scanner=$returncode_scanner_now
			fi
                elif [ "$IS_VIRUS" = 4 ]; then
                        cat $TMPFILE
                        returncode_scanner=3
                        virusfile_avg=`echo $EMAIL`
                        virusname_avg="At least one suspicious object was found by heuristic analysis."
                        echo Infection: avg found \*$virusname_avg\* in \*$virusfile_avg\*
                elif [ "$IS_VIRUS" = 5 ]; then
                        cat $TMPFILE
                        returncode_scanner=3
                        virusfile_avg=`echo $EMAIL`
                        virusname_avg="At least one virus was found by heuristic analysis."
                        echo Infection: avg found \*$virusname_avg\* in \*$virusfile_avg\*
                else
                        cat $TMPFILE
                fi

                AVG_SCANNED=yes
                /bin/rm -f $TMPFILE
                echo
                echo avg done

        else
                echo avg not installed
        fi

   else
        echo avg disabled
   fi

   ##################### AVAST #############
   if [ "x${AVAST_ENABLE}" = "xon" ]; then
     if [ "x$AVAST_INSTALLED" = "xyes" ]; then
 
	echo -----------------------------------------------------------
	echo "scanninge-mail for viruses, starting Avast"
	echo

	#return code 1 is a virus
   	returncode_scanner_now=$returncode_scanner
	TMPFILE=`mktemp $TEMPDIR/avast_outpout_XXXXXX`

        #####################free version of avast  not working for now #############
#        /opt/avast/bin//avast $EMAIL.dir > $TMPFILE || IS_VIRUS=$?



#	if [ $CURRENT_SCAN = "POP3" ]; then
        ##############version with daemon ################## 
	/usr/bin/avastcmd $EMAIL.dir > $TMPFILE || IS_VIRUS=$?
#	fi

#	if [ "$CURRENT_SCAN" = "SMTP" ]; then
#		SIZE_MAIL1=0
#		stat $EMAIL.dir -c%s > /tmp/SIZE_MAIL 2>&1
# 		SIZE_MAIL1=`cat /tmp/SIZE_MAIL`
#		if [ "$SIZE_MAIL1" -le "5000000" ]; then 
#			/usr/bin/avastcmd $EMAIL.dir > $TMPFILE || IS_VIRUS=$?
#		fi
#		/bin/rm -f /tmp/SIZE_MAIL
#	fi


	if [ "$IS_VIRUS" = "1" ]; then 

		returncode_scanner=3
		virusfile_avast=`cat $TMPFILE|grep "infected by" | sed -e 's/.*\/\(.*\) infected by: .*/\1/' | sed 's/.*\.dir\///' | sed 's/\[infected.*//'`
		virusname_avast=`cat $TMPFILE|grep "infected by" | sed -e 's/.*infected by: \(.*\)$/\1/' | sed 's/\]//'`
		if [ "x$virusname_avast" != "x" ]; then
			echo Infection:Avast found \*$virusname_avast\* in \*$virusfile_avast\*
#		else
#			IS_VIRUS=0
#			returncode_scanner=$returncode_scanner_now
		fi
	else
		cat $TMPFILE
		returncode_scanner=$returncode_scanner_now		
	fi
   AVAST_SCANNED=yes
   /bin/rm -f $TMPFILE
   echo
   echo avast done
  else
	echo avast not installed
  fi
else
	echo avast disabled
fi








   if [ "x${FPROT_ENABLE}" = "xon"  ]; then

	if [ "x$FPROT_INSTALLED" = 'xyes' ]; then


	    echo
	    echo
		echo ------------------------------------------------------------------
		echo "scanning email for viruses, starting f-prot..."
		echo 

		# return code 8 is a suspicous file, return code 3 is a virus, 
        # return code 5 is Abnormal termination (scanning did not finish). At least one virus was removed.

                #TIME="\nProcess duration: %e seconds" $TIMEBIN $FPROT -dumb -ai -archive -packed $EMAIL.dir > $TMPDIR/f-prot_output_$UNIQ_FILE.tmp 2>&1 || IS_VIRUS=$?
		returncode_scanner_now=$returncode_scanner
		monit_mail=`cat $EMAIL | grep -E "Copfilter"`
		TMPFILE=`mktemp $TMPDIR/f-prot_output_XXXXXX`
                $FPROT $EMAIL.dir > $TMPFILE || IS_VIRUS=$?
                echo "return code f-prot is $IS_VIRUS"
		if [ "$IS_VIRUS" = 1 ]; then
			cat $TMPFILE
                        returncode_scanner=3
			virusname_fprot=`cat  $TMPFILE | grep -E "Found " | cut -d "<" -f 2 | cut -d ">" -f 1`
			virusfile_fprot=`cat  $TMPFILE | grep -E "Found " | cut -d ">" -f 2 | cut -d "/" -f 12` 
			/bin/mv -f $TMPFILE $TMPDIR/fprot/
			if [ "x$virusname_fprot" != "x" ]; then
				echo Infection: fprot found \*$virusname_fprot\* in \*$virusfile_fprot\*
			else
				IS_VIRUS=0
				returncode_scanner=$returncode_scanner_now
			fi
		elif [ "$IS_VIRUS" = 2 ]; then
			cat $TMPFILE
			if [ "x$monit_mail" = "x" ]; then
				returncode_scanner=3
				virusfile_fprot=`echo $EMAIL`
				virusname_fprot="At least one suspicious object was found."
				echo Infection: fprot found \*$virusname_fprot\* in \*$virusfile_fprot\* 
			fi
#		elif [ "$IS_VIRUS" = 5 ]; then
#			cat $TMPFILE
#			returncode_scanner=3
#			virusfile_fprot=`echo $EMAIL`
#			virusname_fprot="At least one virus was removed"
#			echo Infection: fprot found \*$virusname_fprot\* in \*$virusfile_fprot\*
		else
			cat $TMPFILE
		fi

                FPROT_SCANNED=yes	
		/bin/rm -f $TMPFILE
		echo 
		echo f-prot done
		
	else
		echo fprot not installed
	fi

   else
	echo fprot disabled
   fi
 ############ end size ckek ###########
  
 /bin/rm -f /tmp/SIZE_MAIL
fi
 ######################################

else
  echo virus scanning disabled
fi

# if mail contains virus, there is no need for spam or attachment scanning, saves time
if [ "$returncode_scanner" = "3" ]; then
   echo
   echo
   echo ------------------------------------------------------------------
   echo "virus check"
   echo
   echo "mail contains a virus, will not start spamassassin.."
   echo "deleting temporary files and exiting.."
   cleanup
   cleanup_tmp_files
   if [ "$CURRENT_SCAN" = "SMTP" -a "$SMTP_VIRUS_SCAN_DISCARD" = "on" ]; then
      if [ "x${REJECT_INSTEAD_OF_DISCARD_EMAIL}" = "xon" ];then
          echo "550 Content Rejected: Found virus in email: $virusname_clam"
          echo "550 Content Rejected: Found virus in email: $virusname_clam" >&2
      else
          echo "250 Content Discarded: Found virus in email: $virusname_clam"
          echo "250 Content Discarded: Found virus in email: $virusname_clam" >&2
      fi
      exec 1>&7 7>&-
      cat $LOGFILE
      rm -f $LOGFILE
      exit $returncode_scanner
   else 
      if [ $CURRENT_SCAN = "POP3" -a  $POP3_VIRUS_QUARANTINE = "on" ];then
           echo "quaranting email to  $BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus"
           cp -p $EMAIL $BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus
           if [ "x$virusname_clam" != "x" ]; then
		   echo "$virusname_clam" found by ClamAV >$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
	   fi
	   if [ "x$virusname_avg" != "x" -a "x$virusname_clam" != "x" ]; then
	           echo "$virusname_avg" found by AVG >>$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
	   elif [ "x$virusname_avg" != "x" -a "$virusname_clam" = "x" ]; then
	           echo "$virusname_avg" found by AVG >$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
           fi
	   if [ "x$virusname_fprot" != "x" -a "x$virusname_clam" != "x" -o "x$virusname_avg" != "x" ]; then
		   echo "$virusname_fprot" found by F-Prot >>$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
	   elif [ "x$virusname_fprot" != "x" -a "x$virusname_clam" = "x" -a "x$virusname_avg" = "x" ]; then
		   echo "$virusname_fprot" found by F-Prot >$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
	   fi	    

	   if [ "x$virusname_avast" != "x" -a "x$virusname_fprot" != "x" -o "x$virusname_clam" != "x" -o "x$virusname_avg" != "x" ]; then
		echo "$virusname_avast" found by AVAST >>$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
	   elif [ "x$virusname_avast" != "x" -a "x$virusname_fprot" = "x" -a "virusname_clam" = "x" -a "x$virusname_avg" = "x" ]; then
		  echo "$virusname_avast" found by AVAST >$BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name
	   fi		

	   chmod 644 $BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus.name &>/dev/null
	   chmod 644 $BASEDIR/opt/p3scan/quarantine/${EMAILID}.virus &>/dev/null

      fi
      if [ $CURRENT_SCAN = "SMTP" -a $SMTP_VIRUS_QUARANTINE = "on" ];then
           echo "quaranting email to $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus"
           cp -p $EMAIL $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus
	   chmod 644 $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus &>/dev/null
           if [ "x$virusname_clam" != "x" ]; then
		   echo "$virusname_clam" found by ClamAV >$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name
	   fi
	   if [ "x$virusname_avg" != "x" -a "x$virusname_clam" != "x" ]; then
	           echo "$virusname_avg" found by AVG >>$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name
	   elif [ "x$virusname_avg" != "x" -a "$virusname_clam" = "x" ]; then
	           echo "$virusname_avg" found by AVG >$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name
           fi
	   if [ "x$virusname_fprot" != "x" -a "x$virusname_clam" != "x" -o "x$virusname_avg" != "x" ]; then
		   echo "$virusname_fprot" found by F-Prot >>$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name
	   elif [ "x$virusname_fprot" != "x" -a "x$virusname_clam" = "x" -a "x$virusname_avg" = "x" ]; then
		   echo "$virusname_fprot" found by F-Prot >$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name
	   fi	    
	   if [ "$virusname_avast" != "x" -a "x$virusname_fprot" != "x" -o "x$virusname_clam" != "x" -o "x$virusname_avg" != "x" ]; then
		   echo "$virusname_avast" found by AVAST >>$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name
	   elif [ "x$virusname_avast" != "x" -a "x$virusname_fprot" = "x" -a "x$virusname_clam" = "x" -a "x$virusname_avg" = "x" ]; then
		  echo "$virusname_avast" found by AVAST >$BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name	

	   fi	
           chmod 644 $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus.name &>/dev/null
           chmod 644 $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.virus &>/dev/null
      fi
      if [ $CURRENT_SCAN = "SMTP" ];then
           if [ "x${REJECT_INSTEAD_OF_DISCARD_EMAIL}" = "xon" ];then
               echo "550 Content rejected; Found virus in email: $virusname_clam" >&2
           else
               echo "250 Content accepted; Found virus in email: $virusname_clam" >&2
           fi

           if [ "x${SMTP_VIRUS_NOTIFY_USER}" = "xon" ];then
               echo "sending virus notification"
	       
               MAILSUBJECT=`grep "^Subject:" $EMAIL |sed -e 's/Subject: //; s/
//' | cut -c 1-68`
               
               MAILDATE=`grep "^Date:" $EMAIL |sed -e 's/Date: //; s/
//' | cut -c 1-68`
	       
	       MAIL_VIR_NOTIF=`mktemp $TMPDIR/mail_virus_notification_message_smtp.XXXXXX`
	       
               PROGNAME=ProxSMTP
               VERSIONPROG=`$BASEDIR/opt/proxsmtp/default/sbin/proxsmtpd -v |sed  -n 's/.*version //p'  |sed -e 's/)//'`
               AUTHER=""
               send_virus_notification $EMAIL
            fi
      fi
      if [ $CURRENT_SCAN = "POP3" ];then
           RECIPIENTS=`cat $EMAIL |sed -n '1,/^$/p' | $BASEDIR/opt/tools/bin/formail -c | /usr/bin/awk -f $BASEDIR/opt/tools/bin/extract_email_adresses_to.awk`
           MAILSUBJECT=$SUBJECT
           SERVER=$SERVERIP
           SERVERPORT=":$SERVERPORT"
           CLIENT=$CLIENTIP
           CLIENTPORT=":$CLIENTPORT"
           AUTHOR='by Jack S. Lai <laitcg@cox.net>'
	   MAIL_VIR_NOTIF=`mktemp $TMPDIR/mail_virus_notification_message_pop3.XXXXXX`
           send_virus_notification $EMAIL
      fi

      exec 1>&7 7>&-
      cat $LOGFILE
      echo "mailscanner.sh exit $returncode_scanner"
      rm -f $LOGFILE
      exit $returncode_scanner
   fi
fi


if [ $CURRENT_SCAN = "POP3" -a $POP3_ATTACHMENT_SCAN = "on" -o $CURRENT_SCAN = "SMTP" -a $SMTP_ATTACHMENT_SCAN = "on" ]; then

    echo
    echo
	echo ------------------------------------------------------------------
	echo "scanning for bad attachments, starting renattach..."
	
        #$RENATTACH -a -c $BASEDIR/opt/tools/etc/renattach.conf -e < $EMAIL >  $EMAIL.out
	$RENATTACH -e -c $BASEDIR/opt/tools/etc/renattach.conf < $EMAIL >  $EMAIL.out
        IS_BAD=$?
        if [ "x${IS_BAD}" = "x0" ]; then
           echo "renattach successful, no bad attachments found"
           rm -f $EMAIL.out &>/dev/null
        elif [ "x${IS_BAD}" = "x77" ]; then
           echo "renattach successful, found and processed bad attachments"
           mv $EMAIL.out $EMAIL
        else
           echo "renattach unsuccessful, sending email without attachment scanning"
           rm -f $EMAIL.out &>/dev/null
        fi

        RENATTACH_SCANNED=yes	
        echo
        echo renattach done
    
	if [ "$CURRENT_SCAN" = "SMTP" -a "$SMTP_ATTACHMENT_SCAN_DISCARD" = "on" -a "$IS_BAD" = "77" ]; then
       if [ "x${REJECT_INSTEAD_OF_DISCARD_EMAIL}" = "xon" ];then
           echo "550 Content Rejected: Found bad attachment in email"
           echo "550 Content Rejected: Found bad attachment in email" >&2
       else
           echo "250 Content Discarded: Found bad attachment in email"
           echo "250 Content Discarded: Found bad attachment in email" >&2
       fi
           cleanup
           cleanup_tmp_files
           exec 1>&7 7>&-
           cat $LOGFILE
           rm -f $LOGFILE
	   exit $IS_BAD
	fi

else
  echo attachment scanning disabled
fi





if [ "$CURRENT_SCAN" = "SMTP" ]; then
     #checks if CLIENT is part of our network
     echo
     echo
	 echo ------------------------------------------------------------------
	 echo "Network check"
     echo
#     echo CLIENT is $CLIENT
#     if [ "x${CLIENT}" != "x" ]; then
#          LOCALNET=`perl $BASEDIR/opt/tools/bin/IpInSubnet.pl`
#     fi
     
     if [ "x${LOCALNET}" = "xyes" ]; then
#         echo
#         echo "yes, client is part of our network"
#         echo "no spam scanning will occur"
#         echo
#         cp -p $EMAIL ${EMAIL}_${CURRENT_SCAN}.tmp.$$
#         echo "X-Copfilter:Client is part of our network, skipped SpamAssassin" > ${EMAIL}_${CURRENT_SCAN}.tmp.$$
#         cat $EMAIL >> ${EMAIL}_${CURRENT_SCAN}.tmp.$$
#         mv ${EMAIL}_${CURRENT_SCAN}.tmp.$$ $EMAIL


         if [ "x${WHITELIST_OUTGOING_EMAIL}" = "xon" ]; then
             echo
             echo
	         echo ------------------------------------------------------------------
	         echo "white list editing"
             echo
             echo "since client is part of our network, will try to add recipient to whitelist.."
                echo


             SUBJECT=`grep "^Subject:" $EMAIL |sed -e 's/Subject: //; s/
//' | cut -c 1-68`

             SKIP_WHITELIST=0
             SUBJ_RESULT1=1
             SUBJ_RESULT2=1
             SUBJ_RESULT3=1

             # Out of the Office - Exchange Server
             echo $SUBJECT | egrep -i "^Abwesenheitsnotiz:|^Out of Office AutoReply:"
             SUBJ_RESULT1=$?
             #echo SUBJECT $SUBJECT 
             #echo SUBJ_RESULT1 $SUBJ_RESULT1

             # Out of the Office - Lotus Domino Server
             echo $SUBJECT | grep -i "is out of the office.$"
             SUBJ_RESULT2=$?

             # Mail Delivery Notification - Exchange Server
             echo $SUBJECT | egrep -i "^Delivery Status Notification|^Benachrichtigung.*Zustellstatus"
             SUBJ_RESULT3=$?

             # Mail Delivery Notification - Lotus Notes



             if [ "x${SUBJ_RESULT1}" = "x0" -o "x${SUBJ_RESULT2}" = "x0" -o "x${SUBJ_RESULT3}" = "x0" ]; then
                 SKIP_WHITELIST=1
             fi

             #echo SKIP_WHITELIST $SKIP_WHITELIST

             if [  "x${SKIP_WHITELIST}" = "x0" ]; then
             

		TO_WHITE=""
             	RECIPIENTS2=`echo $RECIPIENTS | sed -e 's/[~!#%&^*?;()<>{}"\/;]//g' -e "s/'//g" -e 's/\[//g' -e 's/\]//g' \
             	      -e 'y/ABCDEFGHIJKLMNOPQRSTUVWXYZ/abcdefghijklmnopqrstuvwxyz/' | sed -e 's/[()<>{}"]//g'`
	        for i in $RECIPIENTS2
        	do
             		ii=`echo $i |sed -n 's/\([a-z0-9][a-z0-9._+-]*@[.a-z0-9_+-]\+[.]\+[a-z][a-z]\+\).*/\1/p'`
		        grep -i "whitelist_from $ii" $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf &>/dev/null
             		RESULT1=$?
             		if [   "x${RESULT1}" != "x0" -o "x${ii}" = "x" ];then
				echo "whitelist_from $ii" >> $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf
				echo "added $ii to whitelist"
             		else
				echo "$i is already in whitelist"
    
                    		#instead of grep'ing for each address, now do a one sort -u at the end
		            	#grep -i "whitelist_from $i" $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf &>/dev/null
       	     		fi
        	done

		# do not sort local_webgui by mailscanner, as we do that in daily cron job
                # sort file
             	# echo "removing double entries in whitelist file"
                # SORTFILE=`mktemp $TMPDIR/cp_spam_whitelist_local_webgui.cf.XXXXXX`
                # sort -u $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf > $SORTFILE
                # cp $SORTFILE $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf
                # rm -f $SORTFILE



             else
             	echo "Email is an AutoReply, no addresses will be added to the whitelist"
             fi


         else 
             echo "whitelisting outgoing email is off"
         fi

      else
         echo "no, client is not part of our network, no whitelist editing"
      fi

fi




if [ "x${LOCALNET}" = "xyes" -a "$CURRENT_SCAN" = "SMTP" ]; then
    if [ "x${DISABLE_SPAM_SCANNING_FROM_OUTGOING_MAIL}" = "xoff" ];then
        LOCALNET=no
        echo "DISABLE_SPAM_SCANNING_FOR_OUTGOING_MAIL_FROM_INTERNAL_NETWORK off"
        echo
        echo "client is part of local network"
        echo
        cp -p $EMAIL ${EMAIL}_${CURRENT_SCAN}.tmp.$$
        echo "X-Copfilter:Client is part of local network" > ${EMAIL}_${CURRENT_SCAN}.tmp.$$
        cat $EMAIL >> ${EMAIL}_${CURRENT_SCAN}.tmp.$$
        mv ${EMAIL}_${CURRENT_SCAN}.tmp.$$ $EMAIL
    else
        echo "DISABLE_SPAM_SCANNING_FOR_OUTGOING_MAIL_FROM_INTERNAL_NETWORK on"
        echo
        echo "yes, client is part of local network"
        echo "no spam scanning will occur"
        echo
        cp -p $EMAIL ${EMAIL}_${CURRENT_SCAN}.tmp.$$
        echo "X-Copfilter:Client is part of local network, skipped Spamassassin" > ${EMAIL}_${CURRENT_SCAN}.tmp.$$
        cat $EMAIL >> ${EMAIL}_${CURRENT_SCAN}.tmp.$$
        mv ${EMAIL}_${CURRENT_SCAN}.tmp.$$ $EMAIL
    fi
fi

IN_WHITE_LIST=0
IN_BLACK_LIST=0

if [ $CURRENT_SCAN = "SMTP" -a "x${SMTP_SPAM_WHITELIST}" = "xon" -o $CURRENT_SCAN = "POP3" -a "x${POP3_SPAM_WHITELIST}" = "xon" ];then

   if [ "x${SENDER}" != "x" -a -f $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf ];then
        RESULT=""
        for i in $SENDER; do
               RESULT=`grep -i -w "$i" $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf`
               echo $RESULT | grep "whitelist_from" &>/dev/null
               IN_WHITE=$?
               if [ "x${IN_WHITE}" = "x0" ];then
                    IN_WHITE_LIST=1
               else
                    IN_WHITE_LIST=0
               fi
               if [ "x${IN_WHITE_LIST}" != "x1" ];then
                        echo $RESULT | grep "blacklist_from" &>/dev/null
                        IN_BLACK=$?
                        if [ "x${IN_BLACK}" = "x0" ];then
                                IN_BLACK_LIST=1
                    else
                                IN_BLACK_LIST=0
                        fi
               fi
        done

       if [ "x${IN_WHITE_LIST}" = "x0" -a "x${IN_BLACK_LIST}" = "x0" ];then
               for i in $SENDER; do
                       SENDERDOM=\*@`echo $SENDER | cut -d @ -f 2`
                       echo "checking for domain part $SENDERDOM in whitelist or blacklist"
                       RESULT=`grep -i -w "$SENDERDOM"$ $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf`
                       echo $RESULT | grep "whitelist_from" &>/dev/null
                       IN_WHITE=$?
                       if [ "x${IN_WHITE}" = "x0" ];then
                               IN_WHITE_LIST=1
                else
                               IN_WHITE_LIST=0
                       fi
                       if [ "x${IN_WHITE_LIST}" != "x1" ];then
                                       echo $RESULT | grep "blacklist_from" &>/dev/null
                                       IN_BLACK=$?
                                       if [ "x${IN_BLACK}" = "x0" ];then
                                       IN_BLACK_LIST=1
                        else
                                       IN_BLACK_LIST=0
                                       fi
                               fi
               done
       fi
        if [ "x${IN_WHITE_LIST}" = "x0" -a "x${IN_BLACK_LIST}" = "x0" ];then
                for i in $SENDER; do
                        SENDERDOM=`echo $SENDER | cut -d @ -f 2`
                        DOMPART=`echo $SENDERDOM | sed 's/^[A-Za-z0-9_]*.@//'`
                                DOTCHECK=`echo $DOMPART | cut -c1`
                                if [ "x${DOTCHECK}" = "x." ]; then
                                      DOMPART=`echo $DOMPART | cut -c2-`
                                      DOMPART=`echo $DOMPART  | sed 's/^[A-Za-z0-9_*]*././'`
                                else
                                      DOMPART=`echo $DOMPART  | sed 's/^[A-Za-z0-9_*]*././'`
                                fi
                        while ( [ $DOMPART != "." -a "$IN_WHITE_LIST" != "1" -a "$IN_BLACK_LIST" != "1" ] ); do
                                echo "checking for domain part *@*$DOMPART in whitelist or blacklist"
                                DOMPARTgrep=`echo $DOMPART |sed -e 's/\./\\\./g'`
                                RESULT=`grep -i -w "\*@\*${DOMPARTgrep}$" $BASEDIR/etc/cp_spam_whitelist/local_webgui.cf`

                                     echo $RESULT | grep "whitelist_from" &>/dev/null
                                     IN_WHITE=$?
                                     if [ "x${IN_WHITE}" = "x0" ];then
                                                 IN_WHITE_LIST=1
                                 else
                                                 IN_WHITE_LIST=0
                                  fi
                                  if [ "x${IN_WHITE_LIST}" != "x1" ];then
                                     echo $RESULT | grep "blacklist_from" &>/dev/null
                                     IN_BLACK=$?
                                     if [ "x${IN_BLACK}" = "x0" ];then
                                         IN_BLACK_LIST=1
                                     else
                                         IN_BLACK_LIST=0
                                     fi
                              fi
                                DOTCHECK=`echo $DOMPART | cut -c1`
                                if [ "x${DOTCHECK}" = "x." ]; then
                                      DOMPART=`echo $DOMPART | cut -c2-`
                                      DOMPART=`echo $DOMPART  | sed 's/^[A-Za-z0-9_*]*././'`
                                else
                                      DOMPART=`echo $DOMPART  | sed 's/^[A-Za-z0-9_*]*././'`
                                fi

                        done
                done
        fi
        SENDERDOM=""
        DOMPART=""

        if [ "x${IN_WHITE_LIST}" = "x1" ];then
             echo "SENDER is in whitelist, skipping spam scanning.."
             echo "X-Copfilter:Sender is in whitelist, skipped SpamAssassin"
             #next line causes errors
             #cat $EMAIL | $BASEDIR/opt/tools/bin/formail -b -f -A \
             #"X-Copfilter:Sender is in whitelist, skipped SpamAssassin"
             cp -p $EMAIL ${EMAIL}_${CURRENT_SCAN}.tmp.$$
             echo "X-Copfilter:Sender is in whitelist, skipped SpamAssassin" > ${EMAIL}_${CURRENT_SCAN}.tmp.$$
             cat $EMAIL >> ${EMAIL}_${CURRENT_SCAN}.tmp.$$
             mv ${EMAIL}_${CURRENT_SCAN}.tmp.$$ $EMAIL

             echo `date "+%b %e %R:%S"` `/bin/hostname -s` "Copfilter_Whitelist spamd clean message (-100.0/${SPAM_SCAN_SCORE_LEVEL}.0) B B B 0" >>$BASEDIR/opt/mail-spamassassin/var/log/spamd.log
        else
             echo "SENDER is not in whitelist, will scan for spam"
        fi

        if [ "x${IN_BLACK_LIST}" = "x1" -a "x${CURRENT_SCAN}" = "xSMTP" ];then
                echo `date "+%b %e %R:%S"` `/bin/hostname -s` "Copfilter_Blacklist spamd identified spam (100.0/${SPAM_SCAN_SCORE_LEVEL}.0) B B B 0" >>$BASEDIR/opt/mail-spamassassin/var/log/spamd.log
                echo "Blacklist email discarding: SENDER is in Copfilter Blacklist, mail will be discarded"
                echo "550 Content Discarded: Sender email address is blacklisted"
                echo "550 Content Discarded: Sender email address is blacklisted" >&2
                IN_WHITE_LIST=0
                exec 1>&7 7>&-
                cat $LOGFILE
                rm -f $LOGFILE
                exit 1
           else
                echo "Blacklist email discarding: This is not a SMTP scan and/or SENDER is not in Copfilter Blacklist"
        fi
   fi

fi

if [ "x${IN_BLACK_LIST}" = "x1" -a $CURRENT_SCAN = "POP3" ];then
    echo "Blacklist email discarding: This is a POP3 Scan, modifying subject to recognize mail as spam"
    SUBJ='*** SPAM *** [Blacklisted] '
    #SUBJECT=$(grep "^Subject:" $EMAIL |sed -e 's/Subject: //; s/
//'  |  cut -c 1-68 | perl -MEncode -ne 'use lib "/var/log/copfilter/default/opt/perl_modules/lib"; print encode("utf8",decode("MIME-Header",$_));')
    SUBJECT=$(sed '/^Subject: */!d; s///;q' $EMAIL | perl -MEncode -ne 'use lib "/var/log/copfilter/default/opt/perl_modules/lib"; print encode("utf8",decode("MIME-Header",$_));')
    SUBJECT="${SUBJ}${SUBJECT}"
    TMPFILE=`mktemp ${EMAIL}_${CURRENT_SCAN}.XXXXXX`
    cat $EMAIL | sed -e 's/^Subject:.*$/Subject: '"$SUBJECT"'/' > $TMPFILE
    mv -f $TMPFILE $EMAIL
    echo "X-Copfilter: Sender is in blacklist, skipped SpamAssassin" > $TMPFILE
    cat $EMAIL >> $TMPFILE
    mv -f $TMPFILE $EMAIL
    echo `date +%a" "%b" "%d" "%X" "%Y` `/bin/hostname -s` "Copfilter_Blacklist identified spam (100.0/${SPAM_SCAN_SCORE_LEVEL}.0) B B B 0" >>$BASEDIR/opt/mail-spamassassin/var/log/spamd.log

   #'Neu Anfang
   SPAM_SCORE=100
   echo "quarantining pop3 email"
   cp -p $EMAIL $BASEDIR/opt/p3scan/quarantine/${EMAILID}.spam
   chmod a+r $BASEDIR/opt/p3scan/quarantine/${EMAILID}.spam &>/dev/null
# if [ "x${POP3_SPAM_QUARANTINE_NOTIF}" = "xon" ];then
#   echo "sending pop3 blacklist quarantine notification instead of tagged spam email"
   #MAILSUBJECT=$(grep "^Subject:" $EMAIL |sed -e 's/Subject: //; s/
//' | cut -c 1-68 | perl -MEncode -ne 'use lib "/var/log/copfilter/default/opt/perl_modules/lib"; print encode("iso8859-1",decode("utf8",$_));')
#   #MAILSUBJECT=$(grep "^Subject:" $EMAIL |sed -e 's/Subject: //; s/
//' | cut -c 1-68 | perl -MEncode -ne 'use lib "/var/log/copfilter/default/opt/perl_modules/lib"; my $enc = find_encoding("utf8"); print $enc->decode($_);')
#   MAILSUBJECT=$(sed '/^Subject: */!d; s///;q' $EMAIL | perl -MEncode -ne 'use lib "/var/log/copfilter/default/opt/perl_modules/lib"; my $enc = find_encoding("utf8"); print $enc->decode($_);')
#   MAILDATE=`grep "^Date:" $EMAIL |sed -e 's/Date: //; s/
//' | cut -c 1-68`
#   RECIPIENTS=`cat $EMAIL |sed -n '1,/^$/p' | $BASEDIR/opt/tools/bin/formail -c | /usr/bin/awk -f $BASEDIR/opt/tools/bin/extract_email_adresses_to.awk`
#   RECIPIENTS2=`echo $RECIPIENTS |cut -c 1-68`

#   echo "From: $SENDER" > $EMAIL
#   echo "To: $RECIPIENTS" >> $EMAIL
#   echo "Subject: [Copfilter] Quarantined BLACKLIST message" >> $EMAIL
#   echo "Date: $MAILDATE" >> $EMAIL
#   echo >> $EMAIL
#   echo >> $EMAIL
#   echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-BLACKLISTED-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=" >> $EMAIL
#   echo >> $EMAIL
#   echo "$CP_spam_qua" >> $EMAIL
#   echo >> $EMAIL
#   echo >> $EMAIL
#   echo "$CP_spam_qua2" >> $EMAIL
#   echo "$CP_spam_qua3" >> $EMAIL

#   echo >> $EMAIL
#   echo "$CP_virusnotif_sender $SENDER" >> $EMAIL
#   echo "$CP_virusnotif_reci $RECIPIENTS2" >> $EMAIL
#   echo "$CP_virusnotif_subj $MAILSUBJECT" >> $EMAIL
#   echo "$CP_virusnotif_date $MAILDATE" >> $EMAIL
#   echo "$CP_virusnotif_file ${EMAILID}.spam" >> $EMAIL
#   echo "$CP_virusnotif_host $HOSTNAME" >> $EMAIL
#   echo "$CP_virusnotif_score [$SPAM_SCORE.0/${POP3_SPAM_SCAN_QUARANTINE_SCORE_LEVEL}.0] [BLACKLIST-Message]" >> $EMAIL
#   echo >> $EMAIL

#   echo $CP_spam_qua4>> $EMAIL
#   echo >> $EMAIL
#   echo "." >> $EMAIL
#   POP3_SANITIZE_SCAN=off
#  fi
#Neu Ende
fi


if [ $CURRENT_SCAN = "POP3" -a $POP3_SPAM_SCAN = "on" -o $CURRENT_SCAN = "SMTP" -a $SMTP_SPAM_SCAN = "on" ]; then


#   if [ "x${SPAMD_ENABLE}" = "xon"  -a  "x${LOCALNET}" != "xyes" -a  "x${IN_WHITE_LIST}" = "x0" -a "x${IN_BLACK_LIST}" = "x0" ]; then
    if [ "x${SPAMD_ENABLE}" = "xon"  -a  "x${LOCALNET}" != "xyes" -a  "x${IN_WHITE_LIST}" = "x0" ]; then
    echo 
    echo 
	echo ------------------------------------------------------------------
	echo "scanning for spam, starting spamassassin (spamc) ..."
	

        #add header to improve SPF checking in spamassassin
        if [ "$CURRENT_SCAN" = "SMTP" -a "x${LOCALNET2}"="xno" ]; then
            echo
            echo "Trying to add a Received Email Header, finding SENDER DOMAIN to use as HELO text in Received Header ..."
            FROMDOM=`echo $SENDER | cut -d@ -f2`
            if [ -n "$FROMDOM" -a -n "$CLIENT" -a -n "$HOSTNAME" ];then
                echo "Adding proxied hop to help SpamAssassin SPF processing..."
                SENDER2=`echo $SENDER | tr -d '\n'`
                HOSTNAME2=`echo $HOSTNAME | tr -d '\n'`
                CLIENT2=`echo $CLIENT | tr -d '\n'`
                FROMDOM2=`echo $FROMDOM | tr -d '\n'`
                echo "X-Envelope-From: $SENDER2"  > ${EMAIL}.out
                echo "added this line to the email header:   X-Envelope-From: $SENDER2"
                echo "Testing is mail can from mail forwarder"
                grep $CLIENT $BASEDIR/etc/mail_forwarders &>/dev/null
                FORWARDCOUNT=$?
                if [ "x${FORWARDCOUNT}" != "x0" ];then
                    echo "Received: from $FROMDOM2 ([$CLIENT2]) by $HOSTNAME2 (Copfilter $VERSION)" >> ${EMAIL}.out
                    echo "added this line to the email header:   Received: from $FROMDOM2 ([$CLIENT2]) by $HOSTNAME2 (Copfilter $VERSION)"
                fi
                cat $EMAIL >> ${EMAIL}.out
                mv ${EMAIL}.out $EMAIL
            else
                echo "Did not find SENDER DOMAIN"
            fi
        fi
        #end add header to improve SPF checking in spamassassin



        #do not use next line since it logs to stderr
        #TIME="\nProcess duration: %e seconds" $TIMEBIN $SPAMASSASSIN -u filter -f < $EMAIL > $EMAIL.out || IS_SPAM=$?
        $SPAMASSASSIN -E -u filter -f < $EMAIL > $EMAIL.out
        IS_SPAM=$?
        mv $EMAIL.out $EMAIL

        SPAM_SCORE=`grep --binary-file=text score $EMAIL |grep X-Spam-Status |sed 's/.* score=\([-0-9]*\).*/\1/g'`
        echo IS_SPAM $IS_SPAM
        echo "SPAM_SCORE $SPAM_SCORE"
        SPAMD_SCANNED=yes	
	echo 
	echo spamassassin done
	echo
	echo


     if [ "x${SPAM_SCORE}" = "x" ];then
         SPAM_SCORE=0
     fi

     SPAM_SCORE=`echo $SPAM_SCORE |sed -e 's/^\([-0-9]*\).*/\1/'`
     echo "changed SPAM_SCORE to $SPAM_SCORE"


	if [ "x${IN_BLACK_LIST}" = "x1" -a $CURRENT_SCAN = "POP3" ];then

         SPAM_SCORE=100
         IS_SPAM=1
        fi

     if [ "$IS_SPAM" -ne "0" ];then
        # email is now spam 


        if [ "$CURRENT_SCAN" = "SMTP" ]; then

           #lt=less than   le=less than or equal
           if [ "x${SMTP_SPAM_SCAN_DISCARD}" = "xon" -a ${SPAM_SCORE} -ge ${SMTP_SPAM_SCAN_DISCARD_SCORE_LEVEL} ]; then
                 if [ "x${REJECT_INSTEAD_OF_DISCARD_EMAIL}" = "xon" ];then
                     echo "550 Content Rejected: Found Spam in email"
                     echo "550 Content Rejected: Found Spam in email" >&2
                 else
                     echo "250 Content Discarded: Found Spam in email"
                     echo "250 Content Discarded: Found Spam in email" >&2
                 fi
                 cleanup
                 cleanup_tmp_files
                  exec 1>&7 7>&-
                  cat $LOGFILE
                  rm -f $LOGFILE
                 exit $IS_SPAM
           fi

           if [ "x${SMTP_SPAM_QUARANTINE}" = "xon" -a ${SPAM_SCORE} -ge ${SMTP_SPAM_SCAN_QUARANTINE_SCORE_LEVEL} ]; then
                  echo "smtp email: quarantining spam email - score was above quarantine score level"
                  echo "..to $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.spam"
                  cp -p $EMAIL $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.spam
		  chmod a+r $BASEDIR/opt/proxsmtp/quarantine/${EMAILID}.spam &>/dev/null
                  if [ "x${REJECT_INSTEAD_OF_DISCARD_EMAIL}" = "xon" ];then
                      echo "550 Content Rejected: Found Spam in email"
                      echo "550 Content Rejected: Found Spam in email" >&2
                  else
                      echo "250 Content Discarded: Found Spam in email"
                      echo "250 Content Discarded: Found Spam in email" >&2
                  fi

                  if [ "x${SMTP_SPAM_QUARANTINE_NOTIFY_USER}" = "xoff" ]; then
                       if [ "x${REJECT_INSTEAD_OF_DISCARD_EMAIL}" = "xon" ];then
                           echo "550 Content Rejected: Found Spam in email"
                           echo "550 Content Rejected: Found Spam in email" >&2
                       else
                           echo "250 Content Discarded: Found Spam in email"
                           echo "250 Content Discarded: Found Spam in email" >&2
                       fi
                       cleanup
                       cleanup_tmp_files
                      exec 1>&7 7>&-
                      cat $LOGFILE
                      rm -f $LOGFILE
                       exit $IS_SPAM
                  fi

           fi 


        else
           if [ "x${POP3_SPAM_QUARANTINE}" = "xon" -a ${SPAM_SCORE} -ge ${POP3_SPAM_SCAN_QUARANTINE_SCORE_LEVEL} ]; then
              echo "quaranting pop3 email"
              cp -p $EMAIL $BASEDIR/opt/p3scan/quarantine/${EMAILID}.spam
	      chmod a+r $BASEDIR/opt/p3scan/quarantine/${EMAILID}.spam &>/dev/null

              if [ "x${POP3_SPAM_QUARANTINE_NOTIF}" = "xon" ]; then 
                echo "sending pop3 spam quarantine notification instead of tagged spam email"
                MAILSUBJECT=`grep "^Subject:" $EMAIL |sed -e 's/Subject: //; s/
//' | cut -c 1-68`
 
                MAILDATE=`grep "^Date:" $EMAIL |sed -e 's/Date: //; s/
//' | cut -c 1-68`
                RECIPIENTS=`cat $EMAIL |sed -n '1,/^$/p' | $BASEDIR/opt/tools/bin/formail -c | /usr/bin/awk -f $BASEDIR/opt/tools/bin/extract_email_adresses_to.awk` 
                RECIPIENTS2=`echo $RECIPIENTS |cut -c 1-68` 

                echo "From: $SENDER" > $EMAIL
                echo "To: $RECIPIENTS" >> $EMAIL
                echo "Subject: [Copfilter] Quarantined Spam message" >> $EMAIL
                echo "Date: $MAILDATE" >> $EMAIL
                echo  >> $EMAIL
                echo >> $EMAIL
                echo "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-" >> $EMAIL
                echo >> $EMAIL
                echo "$CP_spam_qua" >> $EMAIL
                echo >> $EMAIL
                echo >> $EMAIL
                echo "$CP_spam_qua2" >> $EMAIL
                echo "$CP_spam_qua3" >> $EMAIL

                echo >> $EMAIL
                #cat <<EOF >>  $EMAIL
                #
                #Sender:     $SENDER
                #Recipients: $RECIPIENTS2
                #Subject:    $MAILSUBJECT
                #Date:       $MAILDATE
                #Email File: $EMAILID
                #Scanned on: $HOSTNAME
                #Spam score: ${SPAM_SCORE}
                #
                #Please notify your System Administrator if this message is not spam.
                #
                #.
                #EOF
                echo "$CP_virusnotif_sender $SENDER" >> $EMAIL
                echo "$CP_virusnotif_reci $RECIPIENTS2" >> $EMAIL
                echo "$CP_virusnotif_subj $MAILSUBJECT" >> $EMAIL
                echo "$CP_virusnotif_date $MAILDATE" >> $EMAIL
                echo "$CP_virusnotif_file $EMAILID" >> $EMAIL
                echo "$CP_virusnotif_host $HOSTNAME" >> $EMAIL
                echo "$CP_virusnotif_score $SPAM_SCORE" >> $EMAIL
                echo >> $EMAIL

                echo $CP_spam_qua4>> $EMAIL
                echo >> $EMAIL
                echo "." >> $EMAIL
                POP3_SANITIZE_SCAN=off
              fi


           fi
        fi
     fi

   else
	echo spamd disabled, sender in whitelist or blacklist, or client is part of our local network
   fi
else
  echo spam scanning is disabled in webgui
fi



#email sanitizing disabled until p3pmail gets fixed by the author (currently it makes image attachments unreadable)
#if [ $CURRENT_SCAN = "POP3" -a $POP3_SANITIZE_SCAN = "on" -o $CURRENT_SCAN = "SMTP" -a $SMTP_SANITIZE_SCAN = "on" ]; then
#
#        echo ------------------------------------------------------------------
#        echo "sanitizing email ..."
#        echo
#
#        #TIME="\nProcess duration: %e seconds" $TIMEBIN $BIN_ANOMY < $EMAIL > $EMAIL.out
#
#        if [ "x${SANITIZE_METHOD}" = "xANOMY" ]; then
#           $BIN_ANOMY $BASEDIR/opt/anomy/etc/anomy.conf < $EMAIL > $EMAIL.out
#           echo anomy done
#           missing_dot_fix $EMAIL
#        else
#          $BIN_P3PMAIL < $EMAIL > $EMAIL.out
#          echo p3pmail done
#        fi
#
#        mv $EMAIL.out $EMAIL
#
#        echo
#        echo
#
#else
#  echo email sanitizing disabled
#fi



#do not modify email if its already spam
if [ "x$IS_SPAM" = "x0" ];then

	# subject rewrite
	if [ "x$POP3_MAIL_REWRITE_SUBJECT" = "xon" -a "x$CURRENT_SCAN" = "xPOP3" -o "x$SMTP_MAIL_REWRITE_SUBJECT" = "xon" -a "x$CURRENT_SCAN" = "xSMTP" ]; then
        	echo ------------------------------------------------------------------
        	echo "rewriting subject of email ..."
        	echo
	
        	mail_rewrite_subject $EMAIL
	
        	echo "rewriting subject of email done"
        	echo
        	echo
    else
            echo subject rewriting disabled
	fi

	
	
	# add comment to email body
	if [ "x$POP3_MAIL_ADD_BODY" = "xon" -a "x$CURRENT_SCAN" = "xPOP3" -o "x$SMTP_MAIL_ADD_BODY" = "xon" -a "x$CURRENT_SCAN" = "xSMTP" ]; then
        	echo ------------------------------------------------------------------
        	echo "adding copfilter body to email ..."
        	echo
	
        	#grep "\-\-\-\-" $EMAIL &>/dev/null
        	#RESULT1=$?
        	#grep multipart $EMAIL &>/dev/null
        	#RESULT2=$?
        	#if [ "x$RESULT1" = "x0" -a "x$RESULT2" = "x0" ]; then
            # 	$BIN_ANOMY $BASEDIR/opt/anomy/etc/anomy_add_body.conf < $EMAIL > $EMAIL.out
            # 	mv  $EMAIL.out $EMAIL
        	#else
                cat $EMAIL | sed -e '$s/^.$//' |sed -e '$s/^.
$//' > $EMAIL.out
                mv  $EMAIL.out $EMAIL
             	mail_add_body $EMAIL txt $IN_WHITE_LIST
        	#fi
	
            echo EMAIL: $EMAIL
        	missing_dot_fix $EMAIL
	
        	echo "adding copfilter body to email done"
        	echo
        	echo
    else
            echo adding comment to email body disabled
	fi
	
fi



########################## bypass outgoing email larger than 5M
fi
#####################




#fixes some proxsmtpd hangs (for example if body is empty without next lines, this happens when formail is used
#proxsmtpd would terminate with this segmentation fault 
#proxsmtpd: 100001: SERVER: network read operation timed out
#proxsmtpd: 100001: CLIENT > 451 Local Error
#proxsmtpd: 100001: from=tarroc@gmx.net, to=markus.madlener@inode.at, status=FILTER-ERROR
#/etc/rc.d/init.d/copfilter_proxsmtpd: line 424: 23679 Segmentation fault      $PRG_BIN $PRG_BIN_OPTIONS_DEBUG
#haven't investigated yet

#if below fix is not active qmail servers respond with this error:
#451 See http://pobox.com/~djb/docs/smtplf.html.
#Connection closed by foreign host.

if [ $CURRENT_SCAN = "SMTP" ]; then
    echo "SMTP mail, fixing CR/LF"
    cat $EMAIL |sed -e 's/
//g' |sed -e 's/\(.*\)/\1
/' > $EMAIL.out
    mv  $EMAIL.out $EMAIL
elif [ $CURRENT_SCAN = "POP3" ]; then
    echo "POP3 mail"
    #echo "POP3 mail, removing CR"
    #cat $EMAIL |sed -e 's/
//g' > $EMAIL.out
    #mv  $EMAIL.out $EMAIL
fi

##########################
#if ["x${IN_BLACK_LIST}" = "x1" -a $CURRENT_SCAN = "POP3" ]; then
#   rm -f $EMAIL
#   rm -f LOGFILE	
#fi
###########################


echo
echo "mailscanner.sh script finished"
echo

#testing
#cp $EMAIL $TMPDIR/test.$$2
#echo "$TMPDIR/test.$$2"
#cat $EMAIL
#ls -al $EMAIL



#for webgui to show infected emails
chmod -R a+rX $BASEDIR/opt/p3scan/default/var/spool/p3scan &>/dev/null
chmod -R a+rX $BASEDIR/opt/proxsmtp/tmp &>/dev/null

#testing
#cp $EMAIL $TMPDIR
#cat $EMAIL/textfile0 | sed -e 's/^\.//' >$EMAIL/textfile0
#echo TEST >> $EMAIL/textfile0
#echo "." >>  $EMAIL/textfile0

exec 1>&7 7>&-
cat $LOGFILE

rm -f $LOGFILE
cleanup
cleanup_tmp_files
exit $returncode_scanner


